/*
Copyright  2006 - 2007 Annpoint, s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-------------------------------------------------------------------------

NOTE: Reuse requires the following acknowledgement (see also NOTICE):
This product includes Seekafile Server (http://www.seekafile.org) 
developed by Annpoint, s.r.o.

*/


using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using Lucene.Net.Analysis;
using Lucene.Net.Documents;
using Seekafile.Plugin;

namespace Seekafile.Plugin.Default
{
	/// <summary>
	/// Convenience class which provides static methods to extract text from files using installed IFilters
	/// </summary>
	public class IFilterPlugin : IDocumentPlugin
	{
		public IFilterPlugin()
		{
		}

		[DllImport("query.dll", CharSet = CharSet.Unicode)]
		private extern static int LoadIFilter(string pwcsPath, [MarshalAs(UnmanagedType.IUnknown)] object pUnkOuter, ref IFilter ppIUnk);

		private static IFilter loadIFilter(string filename)
		{
			object outer = null;
			IFilter filter = null;

			// Try to load the corresponding IFilter
			int resultLoad = LoadIFilter(filename,  outer, ref filter);
			if (resultLoad != (int) IFilterReturnCodes.S_OK)
			{
				return null;
			}
			return filter;
		}

		public static bool IsParseable(string filename)
		{
			return loadIFilter(filename) != null;
		}

		public static string Extract(string path)
		{
			StringBuilder sb = new StringBuilder();
			IFilter filter = null;

			try
			{
				filter = loadIFilter(path);

				if (filter == null)
					return String.Empty;

				uint i = 0;
				STAT_CHUNK ps = new STAT_CHUNK();

				IFILTER_INIT iflags =
					IFILTER_INIT.CANON_HYPHENS |
					IFILTER_INIT.CANON_PARAGRAPHS |
					IFILTER_INIT.CANON_SPACES |
					IFILTER_INIT.APPLY_CRAWL_ATTRIBUTES |
					IFILTER_INIT.APPLY_INDEX_ATTRIBUTES |
					IFILTER_INIT.APPLY_OTHER_ATTRIBUTES |
					IFILTER_INIT.HARD_LINE_BREAKS |
					IFILTER_INIT.SEARCH_LINKS |
					IFILTER_INIT.FILTER_OWNED_VALUE_OK;

				if (filter.Init(iflags, 0, null, ref i) != (int) IFilterReturnCodes.S_OK)
					throw new Exception("Problem initializing an IFilter for:\n" + path + " \n\n");

				while (filter.GetChunk(out ps) == (int) (IFilterReturnCodes.S_OK))
				{
					if (ps.flags == CHUNKSTATE.CHUNK_TEXT)
					{
						IFilterReturnCodes scode = 0;
						while (scode == IFilterReturnCodes.S_OK)
						{
							uint pcwcBuffer = 65536;
							System.Text.StringBuilder sbBuffer = new System.Text.StringBuilder((int)pcwcBuffer);

							scode = (IFilterReturnCodes) filter.GetText(ref pcwcBuffer, sbBuffer);

							if (pcwcBuffer > 0 && sbBuffer.Length > 0)
							{
								sb.Append(sbBuffer.ToString(0, (int) pcwcBuffer));
								sb.Append(" "); // "\r\n"
							}

						}
					}

				}
			}
			finally
			{
				if (filter != null)
					Marshal.ReleaseComObject(filter);
			}

			return sb.ToString();
		}


		public Document Document(string path)
		{
			FileInfo fi = new FileInfo(path);
			Document doc = new Document();
			doc.Add(new Field("name", fi.Name, true, true, true));
			doc.Add(new Field("fullname", fi.FullName, true, true, false));
	
			DirectoryInfo di = fi.Directory;
			doc.Add(new Field("directparent", di.FullName, true, true, false));
			while (di != null)
			{
				doc.Add(new Field("parent", di.FullName, true, true, false));
				di = di.Parent;
			}
	
			doc.Add(Field.Keyword("created", DateField.DateToString(fi.CreationTime)));
			doc.Add(Field.Keyword("modified", DateField.DateToString(fi.LastWriteTime)));
			doc.Add(Field.Keyword("length", fi.Length.ToString()));
			doc.Add(Field.Keyword("extension", fi.Extension));
	
			string rawText = getBody(path);
			doc.Add(new Field("body", rawText, true, true, true));
			return doc;
		}

		private string getBody(string path)
		{
			return Extract(path);
		}

		public Analyzer Analyzer(string path)
		{
			return null;
		}

		public string Description
		{
			get 
			{ 
				return "Uses IFilter interface to parse the files. It relies on IFilter plugins installed in the system.";
			}
		}

		public string[] Extensions
		{
			get
			{
				return new string[] { ".*" };
			}
		}
	}
}
